#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé le Fri Jan 12 07:57:02 2024

@auteur: david ALBERTO
(www.astrolabe-science.fr)
"""
import matplotlib.pyplot as plt
import matplotlib as mpl # colormap
import numpy as np
import datetime as dt # calcul des rangs du jour dans l'année
import matplotlib.image as image # insertion image
from matplotlib.offsetbox import (OffsetImage, AnnotationBbox) # pour insertion d'une image (logo Creative Commons)


# cmap = plt.get_cmap('cool')
# cmap = plt.get_cmap('Spectral')
cmap = plt.get_cmap('PiYG')
plt.rcParams["font.family"] = "Roboto"

#PARAMETRES Gnomoniques -------------------------------------
lat=np.radians(49.5)
eps=np.radians(23.4)# obliquité de l'écliptique
Hzero=np.arccos(-np.tan(lat)*np.tan(eps))#angle horaire maximal (été)
heure0=round(np.degrees(Hzero)/15)

def azimutcorrige(A, Ah):
    if np.sin(A)<0:
        if np.sin(Ah)>0:
            A=A+np.pi
    else:
       if np.sin(Ah)<0:
        A=A-np.pi
    return A

def declinaison(J):#modélise la déclinaison pour un jour de rang J dans l'année
    long = (2*np.pi*(J-81))/365.2422
    decl = np.arcsin(np.sin(eps)*np.sin(long))
    return decl

def azimut(J):
    decl = declinaison(J)
    az = np.arccos(- np.sin(decl) / np.cos(lat))
    Hz = H0(J)
    az = azimutcorrige(az, Hz)
    return az

def H0(J):
    decl = declinaison(J)
    Hz = np.arccos(-np.tan(lat) * np.tan(decl))
    return Hz

def equinoxes():
    coord_eqP = [81,81]
    coord_eqA = [265, 265]
    x = np.array([np.pi, -np.pi]) / 2
    ax.scatter(x,coord_eqP, c='deeppink', s=20, zorder=2,ec='purple',lw=0.5
               # , label='équinoxe de mars'
               )
    ax.scatter(x,coord_eqA, c='thistle', s=20, zorder=2,ec='darkblue',lw=0.5
               # , label='équinoxe de septembre'
               )

def solsticeEte():
    angle = np.arange(0, 2*np.pi,0.01)
    date = np.ones_like(angle) * 172
    ax.plot(angle, date, c='orangered', zorder=0, lw=1)
    # ax.text(np.pi, 180,"solstice de juin", ha='center', va='center', c='orangered')
    textecourbe('solstice de juin', 0.050, 180, 172, 'orangered')

def solsticeHiver():
    angle = np.arange(0, 2*np.pi,0.01)
    date = np.ones_like(angle) * 355
    ax.plot(angle, date, c='indigo', zorder=0, lw=1)
    textecourbe('solstice de décembre', 0.03, 146.25, 355, 'indigo')

def textecourbe(texte, espace, angle_deg, rayon, couleur):
    """
    Affiche du texte courbe sur un graphe polaire.
    texte : string
    espace : coefficient d'ajustement de l'espace entre les lettres
    angle_deg : angle où écrire le centre du texte (degrés)
    rayon : rayon 
    couleur : couleur
    """
    angle_rad = np.radians(angle_deg)
    for n, lettre in enumerate(texte):
        theta = angle_rad + (len(texte) / 2 - n) * espace
        ax.text(theta, rayon, texte[n], ha='center', va='center',
                rotation=-180+np.degrees(theta), c=couleur,
                bbox=dict(fc='white', ec='None', pad=0))

azimV = np.vectorize(azimut)

fig = plt.figure(figsize=(8,8), facecolor="#C5EDEC", tight_layout=True)

ax = plt.subplot(111, polar=True)
ax.set_theta_zero_location("S")
ax.set_title("Direction du lever et du coucher du Soleil", fontsize='12',
             fontweight='bold', c='darkblue')

annee = 2025
un_janv = dt.datetime(annee, 1, 1)
grad_y = []
for m in range(1,13):
    mois_un = dt.datetime(annee, m, 1)
    njours = mois_un - un_janv
    njours = njours.days
    grad_y.append(njours)

initiales = 'JFMAMJJASOND'
for n, lettre in enumerate(initiales):
    y = grad_y[n]+15
    ax.text(0,y, lettre, ha='center', va='center',
            bbox=dict(fc='white', ec='None', pad=0))

ax.set_yticks(grad_y,[])

vents = ['S', 'SSE', 'SE', 'ESE', 'E', 'ENE', 'NE', 'NNE', 'N', 'NNO', 'NO', 'ONO', 'O', 'OSO', 'SO', 'SSO']

ax.set_xticks(np.radians(np.arange(0,360,22.5)), vents)
ax.set_ylim(0,366)

liste_lat = np.radians(np.arange(0,70,10))
norm = mpl.colors.Normalize(vmin=min(liste_lat),vmax=max(liste_lat))

for lat in liste_lat:
    Y = np.arange(1,366)
    X = azimV(Y)
    lat_deg = str(round(np.degrees(lat))) + '°'
    ax.plot(X,Y, c=cmap(norm(lat)), label = lat_deg)
    ax.plot(-X,Y, c=cmap(norm(lat)))

equinoxes()
solsticeEte()
solsticeHiver()

ax.legend(title='latitudes', loc='upper center', fontsize=8)

textecourbe('lever', 0.035, 101, 320, 'darkslateblue')
textecourbe('coucher', 0.035, -101, 320, 'darkslateblue')

fig.text(0.94, 0.02, "D. Alberto (www.astrolabe-science.fr",
          transform = fig.transFigure, rotation=90, fontsize=8)

# insertion du logo CC :
logo = image.imread("../by-nc-sa.png")
imagebox = OffsetImage(logo, zoom = 0.1)
ab = AnnotationBbox(imagebox, (0.15, 355), frameon = True)
ax.add_artist(ab)

fig.savefig("amplitudeOrtiveSoleil.pdf")
fig.savefig("amplitudeOrtiveSoleil.png", dpi=200)
